#!/usr/bin/python
# -*- encoding: utf-8 -*-


# Priklad ovladani obvodu PCA9685: 
# Pomal blikn LED s vyuitm vnjho genertoru (HEF4060)
# propojka extCLK je propojena na vystup obvodu 4060, na kterem je frekvence 125kHz

import time
import smbus


bus = smbus.SMBus(1)   # novejsi varianta RasPi (512MB)
#bus = smbus.SMBus(0)   # starsi varianta RasPi (256MB)

addr = 0x40            # I2C adresa obvodu PCA9685 (kdyz jsou vsechny adresovaci vstupy v "0", je adresa 0x40)


gen=125000                                     # frekvence vnejsiho generatoru HEF4060 (125kHz)
frekvence = 3                                  # pozadovana frekvence vystupu v Hz
strida = 0.5                                   # sirka pulzu 0.5 = 50%
kanal = 1                                     # cislo kanalu 0 az 15

n = int((gen /frekvence) / 4096)-1             # prepocet frekvence na pozadovanou konstantu prescaleru (orezava)
nastavena_frekv = gen / (4096.0 * (n+1))       # zpetny vypocet skutecne nastavene frekvence

sirka_H = int(4095 * strida ) & 0xFFF          # sirka impulzu se nasatvuje v rozsahu 0 az 4095 pro 0% az 100%

adr_kan = 6 + (kanal * 4)                      # kazdy kanal se ridi pomomci 4 bajtu. Nulty kanal zacina na adrese 6

dolnibajt_H = int(sirka_H % 256)               # prepocet 12-bitove sirky impulzu na horni a dolni bajt
hornibajt_H = int(sirka_H / 256)

print "Skutecne nastavena frekvence:  " + str(nastavena_frekv)  + " (prescaler (0 az 255): " + str(n) + ")"
print "Skutecna sirka signalu (0 az 4095):  " + str(sirka_H) + "   (" + str(sirka_H / 40.95) + "%)"



# inicializace PWM
bus.write_byte_data(addr,0x00,0b00010000)  # SLEEP - zastaveni generatoru
time.sleep(0.001)
bus.write_byte_data(addr,0x00,0b01010000)  # SLEEP s nastavenim "EXT CLK"
bus.write_byte_data(addr,0xFE,n)     # nastaveni frekvence prescaleru
bus.write_byte_data(addr,0x01,0x04)  # Logika vystupu: Vystup je aktivni pri "1" ("H" = Vdd ; "L" = GND)
bus.write_byte_data(addr,0x00,0b01000000)  # zruseni SLEEP


# nastaveni parametru jednoho kanalu
bus.write_byte_data(addr, adr_kan,     0x00) # prechod signalu z "0" na "1" v case "T + nula"  8xLSB
bus.write_byte_data(addr, adr_kan + 1, 0x00) #                                                 4xMSB
bus.write_byte_data(addr, adr_kan + 2, dolnibajt_H) # cas prechodu signalu z "1" na "0" 8xLSB
bus.write_byte_data(addr, adr_kan + 3, hornibajt_H) #                                   4xMSB
  
  
# spusteni generatoru na 10 sekund    
bus.write_byte_data(addr,0x00,0x00)  # spust PWM generator
time.sleep(10)                       # 10 sekund generovani signalu
bus.write_byte_data(addr,0x00,0x10)  # zastav PWM generator  (bez zastaveni pobezi neustale nezavisle na RasPi)
  

